<?php

namespace UnicaenDbImport\CodeGenerator;

use Doctrine\DBAL\DBALException;
use UnicaenApp\Exception\RuntimeException;
use UnicaenDbImport\Domain\DestinationInterface;
use UnicaenDbImport\Domain\ResultInterface;
use UnicaenDbImport\Domain\SourceInterface;

/**
 * Contrat à honorer pour les générateurs de code SQL.
 *
 * @package UnicaenDbImport\CodeGenerator\Common
 */
interface CodeGeneratorInterface
{
    /**
     * @param string $tableName
     * @return string
     */
    public function generateSQLForTableExistenceCheck($tableName);

    /**
     * @param array $result
     * @return bool
     */
    public function convertTableExistenceCheckResultToBoolean(array $result);

    /**
     * @param string $tableName
     * @param int $columnsCount
     * @return string
     */
    public function generateSQLForHistoColumnsValidationInTable(string $tableName, int &$columnsCount);

    /**
     * @param string $tableName
     * @param array $result
     * @param int $columnCount
     * @return null|RuntimeException
     */
    public function convertHistoColumnsValidationBadResultToException($tableName, array $result, int $columnCount);

    /**
     * @param string $tableName
     * @param string $sourceCodeColumn
     * @param int $columCount
     * @return string
     * @codeCoverageIgnore Car simple délégation
     */
    public function generateSQLForSourceColumnsValidationInTable(string $tableName, string $sourceCodeColumn, int &$columCount);

    /**
     * @param string $tableName
     * @param string $sourceCodeColumn
     * @param array $result
     * @param int $columCount
     * @return null|RuntimeException
     * @codeCoverageIgnore Car simple délégation
     */
    public function convertSourceColumnsValidationBadResultToException(string $tableName, string $sourceCodeColumn, array $result, int $columCount);

    /**
     * @param SourceInterface $source
     * @return string
     */
    public function generateSQLForSelectFromSource(SourceInterface $source);

    /**
     * @param SourceInterface      $source
     * @param DestinationInterface $destination
     * @param string               $intermediateTable
     * @return string
     */
    public function generateSQLForIntermediateTableCreation(SourceInterface $source,
                                                            DestinationInterface $destination,
                                                            $intermediateTable);

    /**
     * @param string $tableName
     * @param array $columnsValues
     * @param string $sourceCode
     * @param string|null|false $idColumnSequence
     * @return string
     */
    public function generateSQLForInsertIntoTable(
        string $tableName,
        array $columnsValues,
        string $sourceCode,
        $idColumnSequence = null);

    /**
     * @param string $logTableName
     * @return string
     */
    public function generateSQLForImportLogTableCreation(string $logTableName);

    /**
     * @param string $tableName
     * @return string
     */
    public function generateSQLForIntermmediateTableDrop($tableName);

    /**
     * @param string $tableName
     * @return string
     */
    public function generateSQLForClearTable($tableName);

    /**
     * @param SourceInterface      $source
     * @param DestinationInterface $destination
     * @param string               $intermediateTable
     * @return string
     * @throws DBALException
     */
    public function generateSQLForDiffViewCreation(SourceInterface $source, DestinationInterface $destination, $intermediateTable);

    /**
     * @param SourceInterface      $source
     * @param DestinationInterface $destination
     * @return string
     */
    public function generateSQLForDiffViewSelect(SourceInterface $source, DestinationInterface $destination);

    /**
     * @param string $operation
     * @param SourceInterface $source
     * @param DestinationInterface $destination
     * @return string[]
     */
    public function generateSQLForDestinationUpdate(
        $operation,
        SourceInterface $source,
        DestinationInterface $destination);

    /**
     * @param ResultInterface $result
     * @param string          $importLogTable
     * @return string
     */
    public function generateSQLForInsertResultIntoLogTable(ResultInterface $result, string $importLogTable);
}