<?php

namespace UnicaenDbImport\CodeGenerator\Oracle\Helper;

use UnicaenDbImport\Domain\Operation;
use UnicaenDbImport\Platforms\OraclePlatform;

/**
 * Génération de code SQL autour des tables.
 *
 * Version Oracle.
 *
 * @author Unicaen
 */
class TableHelper extends \UnicaenDbImport\CodeGenerator\Helper\TableHelper
{
    /**
     * Helper constructor.
     */
    public function __construct()
    {
        $this->platform = new OraclePlatform();
    }

    /**
     * @inheritDoc
     */
    public function generateSQLForIntermediateTableCreation($destinationTable, $sourceCodeColumn, $columns, $intermediateTable)
    {
        // les ; perturbent le driver Oracle donc on met ça dans un bloc BEGIN...END
        return
            'BEGIN ' . PHP_EOL .
            parent::generateSQLForIntermediateTableCreation($destinationTable, $sourceCodeColumn, $columns, $intermediateTable) . PHP_EOL .
            'END;';
    }

    /**
     * @param string $diffSourceTableName
     * @param string $destinationTable
     * @param string $sourceCodeColumn
     * @param array  $columns
     * @return string
     */
    public function generateSQLForDiffViewCreation($diffSourceTableName, $destinationTable, $sourceCodeColumn, array $columns)
    {
        $res = '';
        $res .= $this->generateDiffViewCreationSQLSnippet($diffSourceTableName, $destinationTable, $sourceCodeColumn, $columns) . PHP_EOL;

        return $res;
    }

    /**
     * @param string $diffSourceTableName
     * @param string $destinationTable
     * @param string $sourceCodeColumn
     * @param array  $columns
     * @return string
     */
    protected function generateDiffViewCreationSQLSnippet($diffSourceTableName, $destinationTable, $sourceCodeColumn, array $columns)
    {
        $name = $this->generateDiffViewName($destinationTable);

        $res = '';
        $res .= "CREATE OR REPLACE VIEW $name AS " . PHP_EOL;
        $res .= $this->indent(4, $this->generateDiffViewBodySelectSQLSnippet($diffSourceTableName, $destinationTable, $sourceCodeColumn, $columns)) . PHP_EOL;

        return $res;
    }

    /**
     * {@inheritDoc}
     */
    protected function generateDiffViewDeletionSQLSnippet($destinationTable)
    {
        $name = $this->generateDiffViewName($destinationTable);

        return "DROP VIEW $name";
    }

    /**
     * {@inheritDoc}
     */
    public function generateSQLForUpdateOperationInDestinationTable($destinationTable, $sourceCodeColumn, array $columns, $where = null)
    {
        return $this->generateSQLUpdate(Operation::OPERATION_UPDATE, $destinationTable, $sourceCodeColumn, $columns, $where);
    }

    /**
     * {@inheritDoc}
     */
    public function generateSQLForUndeleteOperationInDestinationTable($destinationTable, $sourceCodeColumn, array $columns, $where = null)
    {
        return $this->generateSQLUpdate(Operation::OPERATION_UNDELETE, $destinationTable, $sourceCodeColumn, $columns, $where);
    }

    /**
     * {@inheritDoc}
     */
    public function generateSQLForDeleteOperationInDestinationTable($destinationTable, $sourceCodeColumn, array $columns, $where = null)
    {
        return $this->generateSQLUpdate(Operation::OPERATION_DELETE, $destinationTable, $sourceCodeColumn, $columns, $where);
    }

    /**
     * @param string $operation
     * @param string $destinationTable
     * @param string $sourceCodeColumn
     * @param array $columns
     * @param string $where
     * @return string
     */
    protected function generateSQLUpdate($operation, $destinationTable, $sourceCodeColumn, array $columns, $where = null)
    {
        $diffViewName = $this->generateDiffViewName($destinationTable);
        $setters = $this->generateSQLUpdateSetters($operation, $columns);
        $where = "diff.operation = '$operation' AND d.$sourceCodeColumn = diff.$sourceCodeColumn";

        // instruction de mise à jour de la table destination
        $markup = '';
        $markup .= "MERGE INTO $destinationTable d USING (SELECT * FROM $diffViewName) diff " . PHP_EOL;
        $markup .= "ON ($where) " . PHP_EOL;
        $markup .= "WHEN MATCHED THEN UPDATE SET " . PHP_EOL;
        $markup .= $setters;

        return $markup;
    }

    /**
     * @param string $sequenceName
     * @return string
     */
    public function generateSQLForSequenceNextVal(string $sequenceName)
    {
        return $sequenceName . '.nextval';
    }

    /**
     * @inheritDoc
     */
    public function generateSQLForCastToString(string $columnName)
    {
        return $columnName . "||''";
    }
}