<?php

namespace UnicaenDbImport\CodeGenerator\PostgreSQL\Helper;

use Doctrine\DBAL\Platforms\PostgreSqlPlatform;
use UnicaenDbImport\Domain\Operation;

/**
 * Génération de code SQL autour des tables.
 *
 * Version PostgreSQL.
 *
 * @author Unicaen
 */
class TableHelper extends \UnicaenDbImport\CodeGenerator\Helper\TableHelper
{
    /**
     * Helper constructor.
     */
    public function __construct()
    {
        $this->platform = new PostgreSqlPlatform();
    }

    /**
     * {@inheritDoc}
     */
    protected function generateDiffViewDeletionSQLSnippet($destinationTable)
    {
        $name = $this->generateDiffViewName($destinationTable);

        return "DROP VIEW IF EXISTS $name";
    }

    /**
     * {@inheritDoc}
     */
    public function generateSQLForUpdateOperationInDestinationTable($destinationTable, $sourceCodeColumn, array $columns, $where = null)
    {
        return $this->generateSQLUpdate(Operation::OPERATION_UPDATE, $destinationTable, $sourceCodeColumn, $columns, $where);
    }

    /**
     * {@inheritDoc}
     */
    public function generateSQLForUndeleteOperationInDestinationTable($destinationTable, $sourceCodeColumn, array $columns, $where = null)
    {
        return $this->generateSQLUpdate(Operation::OPERATION_UNDELETE, $destinationTable, $sourceCodeColumn, $columns, $where);
    }

    /**
     * {@inheritDoc}
     */
    public function generateSQLForDeleteOperationInDestinationTable($destinationTable, $sourceCodeColumn, array $columns, $where = null)
    {
        return $this->generateSQLUpdate(Operation::OPERATION_DELETE, $destinationTable, $sourceCodeColumn, $columns, $where);
    }

    /**
     * @param string $operation
     * @param string $destinationTable
     * @param string $sourceCodeColumn
     * @param array $columns
     * @param string $where
     * @return string
     */
    protected function generateSQLUpdate($operation, $destinationTable, $sourceCodeColumn, array $columns, $where = null)
    {
        $diffViewName = $this->generateDiffViewName($destinationTable);
        $setters = $this->generateSQLUpdateSetters($operation, $columns);
        $wheres = "diff.operation = '$operation' AND d.$sourceCodeColumn = diff.$sourceCodeColumn";
        if ($where) {
            $wheres .= sprintf(" AND (%s)", $where);
        }

        // instruction de mise à jour de la table destination
        $markup = '';
        $markup .= "UPDATE $destinationTable d SET " . PHP_EOL;
        $markup .= $setters . PHP_EOL;
        $markup .= "FROM (SELECT * FROM $diffViewName) AS diff " . PHP_EOL;
        $markup .= "WHERE $wheres";

        return $markup;
    }

    /**
     * @param string $sequenceName
     * @return string
     */
    public function generateSQLForSequenceNextVal(string $sequenceName)
    {
        return "nextval('" . $sequenceName . "')";
    }

    /**
     * @inheritDoc
     */
    public function generateSQLForCastToString(string $columnName)
    {
        return $columnName . '::varchar';
    }
}