<?php

namespace UnicaenDbImport\Controller;

use UnicaenDbImport\Entity\Db\Service\ImportLog\ImportLogServiceAwareTrait;
use UnicaenDbImport\Service\Traits\ImportServiceAwareTrait;
use Laminas\Http\Response;
use Laminas\Mvc\Controller\AbstractActionController;
use Laminas\View\Model\JsonModel;

/**
 * @method \Laminas\Mvc\Plugin\FlashMessenger\FlashMessenger flashMessenger()
 */
class ImportController extends AbstractActionController
{
    use ImportServiceAwareTrait;
    use ImportLogServiceAwareTrait;

    /**
     * Listage des imports.
     *
     * @return array
     */
    public function indexAction(): array
    {
        $imports = $this->importService->getImports();

        $importLogs = [];
        foreach ($imports as $import) {
            $importLog = $this->importLogService->findLastLogForImport($import);
            $importLogs[$import->getName()] = $importLog;
        }

        return [
            'imports' => $imports,
            'importLogs' => $importLogs,
        ];
    }

    /**
     * Détails d'une synchro.
     *
     * @return array
     */
    public function voirAction(): array
    {
        $name = $this->params('name');
        $import = $this->importService->getImportByName($name);
        $importLog = $this->importLogService->findLastLogForImport($import);

        return [
            'import' => $import,
            'importLog' => $importLog,
        ];
    }

    /**
     * Lancement d'un import.
     *
     * @throws \Exception
     */
    public function lancerAction(): Response
    {
        $name = $this->params('name');
        $synchro = $this->importService->getImportByName($name);

        $result = $this->importService->runImport($synchro);

        $startDate = $result->getStartDate();
        $endDate = $result->getEndDate();
        $diffDate = date_diff($startDate, $endDate);
        $duration = $diffDate->format('%i min %s sec.');

        if ($exception = $result->getFailure()) {
            throw $exception;
        }
        if ($result->hasExceptionInResults()) {
            $this->flashMessenger()->addErrorMessage("L'import a rencontré un problème ($duration).");
        } else {
            $this->flashMessenger()->addSuccessMessage("Import réalisé avec succès ($duration).");
        }
        $this->flashMessenger()->addInfoMessage(str_replace(PHP_EOL, '<br>', $result->toString(true)));

        return $this->redirect()->toRoute('unicaen-db-import/import/voir', ['name' => $synchro->getName()]);
    }

    /**
     * @return \Laminas\View\Model\JsonModel
     */
    public function lastLogAction(): JsonModel
    {
        $name = $this->params('name');
        $import = $this->importService->getImportByName($name);
        $importLog = $this->importLogService->findLastLogForImport($import);

        return new JsonModel([
            'importLog' => $importLog,
        ]);
    }
}