<?php

namespace UnicaenDbImport\Domain;

use Assert\AssertionChain;
use UnicaenDbImport\Config\ConfigException;
use Laminas\Stdlib\Parameters;

class Synchro implements SynchroInterface
{
    const V_DIFF_UPDATED_COLUMN_PREFIX = 'U_';

    const V_DIFF_SRC_COLUMN_PREFIX = 'S_';
    const V_DIFF_DEST_COLUMN_PREFIX = 'D_';

    /**
     * @var Parameters
     */
    protected $config;

    /**
     * @param array $config
     * @return static
     */
    static public function fromConfig(array $config)
    {
        $inst = new static($config);

        try {
            $inst->validateConfig();
        } catch (\Exception $e) {
            if (! $e instanceof ConfigException) {
                $e = new ConfigException($e->getMessage(), 0, $e);
            }
            throw $e;
        }

        return $inst;
    }

    protected function __construct(array $config)
    {
        $this->config = new Parameters($config);
    }

    protected function validateConfig()
    {
        (new AssertionChain($this->config->get($key = 'source'), "Une instance de " . SourceInterface::class . " est requise pour la clé suivante: $key"))
            ->notEmpty()
            ->isInstanceOf(SourceInterface::class);

        (new AssertionChain($this->config->get($key = 'destination'), "Une instance de " . DestinationInterface::class . " est requise pour la clé suivante: $key"))
            ->notEmpty()
            ->isInstanceOf(DestinationInterface::class);

        if ($this->config->offsetExists($key = 'intermediate_table')) {
            $intermediateTable = $this->config->get($key = 'intermediate_table');
            (new AssertionChain($intermediateTable, "Une string non vide est requise pour la clé facultative suivante: $key"))
                ->notEmpty()
                ->string();
        }
    }

    /**
     * @return string
     */
    public function __toString()
    {
        return $this->getName();
    }

    /**
     * @return string
     */
    public function getName()
    {
        if ($name = $this->config->get('name')) {
            return $name;
        }

        return $this->getSource()->getName() . ' ==> ' . $this->getDestination()->getName();
    }

    /**
     * @return Source
     */
    public function getSource()
    {
        return $this->config->get('source');
    }

    /**
     * @return Destination
     */
    public function getDestination()
    {
        return $this->config->get('destination');
    }
}