<?php

namespace UnicaenDbImport\Entity\Db;

use DateTime;

abstract class AbstractImportLog
{
    const TYPE_IMPORT = 'import';
    const TYPE_SYNCHRO = 'synchro';

    /**
     * @var int
     */
    protected $id;

    /**
     * @var string
     */
    protected $type;

    /**
     * @var string
     */
    protected $name;

    /**
     * @var boolean
     */
    protected $success;

    /**
     * @var string
     */
    protected $log;

    /**
     * @var DateTime
     */
    protected $startedOn;

    /**
     * @var DateTime
     */
    protected $endedOn;

    /**
     * @var string
     */
    protected $importHash;

    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @return string
     */
    public function getType(): string
    {
        return $this->type;
    }

    /**
     * @param string $type
     * @return self
     */
    public function setType(string $type): self
    {
        $this->type = $type;
        return $this;
    }

    /**
     * @return string
     */
    public function getName(): string
    {
        return $this->name;
    }

    /**
     * @param string $name
     * @return self
     */
    public function setName(string $name): self
    {
        $this->name = $name;
        return $this;
    }

    /**
     * @return bool
     */
    public function isSuccess(): bool
    {
        return $this->success;
    }

    /**
     * @param bool $success
     * @return self
     */
    public function setSuccess(bool $success): self
    {
        $this->success = $success;
        return $this;
    }

    /**
     * @return string
     */
    public function getLog(): string
    {
        return $this->log;
    }

    /**
     * @param string $log
     * @return self
     */
    public function setLog(string $log): self
    {
        $this->log = $log;
        return $this;
    }

    /**
     * @return DateTime
     */
    public function getStartedOn(): DateTime
    {
        return $this->startedOn;
    }

    /**
     * @return string
     */
    public function getStartedOnToString(): string
    {
        return $this->startedOn->format('d/m/Y H:i:s');
    }

    /**
     * @param DateTime $startedOn
     * @return self
     */
    public function setStartedOn(DateTime $startedOn): self
    {
        $this->startedOn = $startedOn;
        return $this;
    }

    /**
     * @return DateTime
     */
    public function getEndedOn(): DateTime
    {
        return $this->endedOn;
    }

    /**
     * @return string
     */
    public function getEndedOnToString(): string
    {
        return $this->endedOn->format('d/m/Y H:i:s');
    }

    /**
     * @param DateTime $endedOn
     * @return self
     */
    public function setEndedOn(DateTime $endedOn): self
    {
        $this->endedOn = $endedOn;
        return $this;
    }

    /**
     * @return string|null
     */
    public function getImportHash(): ?string
    {
        return $this->importHash;
    }

    /**
     * @param string $importHash
     * @return self
     */
    public function setImportHash(string $importHash): self
    {
        $this->importHash = $importHash;
        return $this;
    }
}
