<?php

namespace UnicaenDbImport;

use Doctrine\DBAL\Connection;
use Doctrine\DBAL\DBALException;
use Doctrine\DBAL\Driver\ResultStatement;
use Doctrine\DBAL\Query\QueryBuilder;

/**
 * @codeCoverageIgnore
 *
 * @author Unicaen
 */
class QueryExecutor
{
    /**
     * @param string $sql
     * @param Connection $connection
     * @return ResultStatement
     * @throws \Doctrine\DBAL\Exception
     */
    private function executeQuery($sql, Connection $connection): ResultStatement
    {
        return $connection->executeQuery($sql);
    }

    /**
     * @param QueryBuilder $qb
     * @param int          $firstResult
     * @param int          $maxResults
     * @return array
     */
    public function fetchPartial(QueryBuilder $qb, $firstResult, $maxResults)
    {
        //
        // Attention, brouillon!
        //

        $qb
            ->setFirstResult($firstResult)
            ->setMaxResults($maxResults);

        $statement = $qb->execute();

        return $statement->fetchAll(\PDO::FETCH_ASSOC);
    }

    /**
     * @param string $sql
     * @param Connection $connection
     * @return array
     * @throws \Doctrine\DBAL\Exception
     */
    public function fetchAllAsGenerator(string $sql, Connection $connection)
    {
        $statement = $this->executeQuery($sql, $connection);

        $rows = [];
        while ($row = $statement->fetchAssociative()) {
            $rows[] = $row;
            yield $row;
        }

        return $rows;
    }

    /**
     * @param string $sql
     * @param Connection $connection
     * @return array
     * @throws \Doctrine\DBAL\Exception
     */
    public function fetchAll(string $sql, Connection $connection): array
    {
        $statement = $this->executeQuery($sql, $connection);

        return $statement->fetchAllAssociative();
    }

    /**
     * @param string $sql
     * @param Connection $connection
     * @return array|null
     * @throws \Doctrine\DBAL\Exception
     */
    public function fetch(string $sql, Connection $connection): ?array
    {
        $statement = $this->executeQuery($sql, $connection);

        return $statement->fetchAssociative() ?: null;
    }

    /**
     * @param string $sql
     * @param Connection $connection
     * @return int
     * @throws DBALException
     */
    public function exec(string $sql, Connection $connection): int
    {
        return $connection->executeStatement($sql);
    }
}